@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2024	PgPool Global Development Group
+ * Copyright (c) 2003-2020	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -165,8 +165,6 @@ pool_start_query(POOL_QUERY_CONTEXT * query_context, char *query, int len, Node
 			query_context->temp_cache = pool_create_temp_query_cache(query);
 		pool_set_query_in_progress();
 		query_context->skip_cache_commit = false;
-		query_context->atEnd = false;
-		query_context->partial_fetch = false;
 		session_context->query_context = query_context;
 		MemoryContextSwitchTo(old_context);
 	}
@@ -238,31 +236,14 @@ pool_setall_node_to_be_sent(POOL_QUERY_CONTEXT * query_context)
 		if (private_backend_status[i] == CON_UP ||
 			(private_backend_status[i] == CON_CONNECT_WAIT))
 		{
-			if (SL_MODE)
+			/*
+			 * In streaming replication mode, if the node is not primary node
+			 * nor load balance node, there's no point to send query.
+			 */
+			if (SL_MODE && !pool_config->statement_level_load_balance &&
+				i != PRIMARY_NODE_ID && i != sc->load_balance_node_id)
 			{
-				/*
-				 * If load balance mode is disabled, only send to the primary node.
-				 * If primary node does not exist, send to the main node.
-				 */
-				if (!pool_config->load_balance_mode)
-				{
-					if (i == PRIMARY_NODE_ID ||
-						(PRIMARY_NODE_ID < 0 && MAIN_NODE_ID == i))
-					{
-						query_context->where_to_send[i] = true;
-						break;
-					}
-					continue;
-				}
-				else
-					/*
-					 * If the node is not primary node nor load balance node,
-					 * there's no point to send query except statement level
-					 * load balance is enabled.
-					 */
-					if (!pool_config->statement_level_load_balance &&
-						i != PRIMARY_NODE_ID && i != sc->load_balance_node_id)
-						continue;
+				continue;
 			}
 			query_context->where_to_send[i] = true;
 		}
@@ -363,16 +344,7 @@ pool_virtual_main_db_node_id(void)
 	sc = pool_get_session_context(true);
 	if (!sc)
 	{
-		/*
-		 * We used to return REAL_MAIN_NODE_ID here.  Problem with it is, it
-		 * is possible that REAL_MAIN_NODE_ID could be changed
-		 * anytime. Suppose REAL_MAIN_NODE_ID == my_main_node_id == 1. Then
-		 * due to failback, REAL_MAIN_NODE_ID is changed to 0. Then
-		 * MAIN_CONNECTION(cp) will return NULL and any reference to it will
-		 * cause segmentation fault. To prevent the issue we should return
-		 * my_main_node_id instead.
-		 */
-		return my_main_node_id;
+		return REAL_MAIN_NODE_ID;
 	}
 
 	if (sc->in_progress && sc->query_context)
@@ -415,7 +387,7 @@ pool_virtual_main_db_node_id(void)
 	}
 
 	/*
-	 * No query context exists.  If in streaming replication mode, returns primary node
+	 * No query context exists.  If in native replication mode, returns primary node
 	 * if exists.  Otherwise returns my_main_node_id, which represents the
 	 * last REAL_MAIN_NODE_ID.
 	 */
@@ -479,7 +451,7 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 	else if (MAIN_REPLICA && query_context->is_multi_statement)
 	{
 		/*
-		 * If we are in streaming replication mode and we have multi statement query,
+		 * If we are in native replication mode and we have multi statement query,
 		 * we should send it to primary server only. Otherwise it is possible
 		 * to send a write query to standby servers because we only use the
 		 * first element of the multi statement query and don't care about the
@@ -512,21 +484,7 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 		/* Should be sent to both primary and standby? */
 		else if (dest == POOL_BOTH)
 		{
-			if (is_tx_started_by_multi_statement_query())
-			{
-				/*
-				 * If we are in an explicit transaction and the transaction
-				 * was started by a multi statement query, we should send
-				 * query to primary node only (which was supposed to be sent
-				 * to all nodes) until the transaction gets committed or
-				 * aborted.
-				 */
-				pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
-			}
-			else
-			{
-				pool_setall_node_to_be_sent(query_context);
-			}
+			pool_setall_node_to_be_sent(query_context);
 		}
 		else if (pool_is_writing_transaction() &&
 				 pool_config->disable_load_balance_on_write == DLBOW_ALWAYS)
@@ -569,6 +527,34 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 					 * Load balance if possible
 					 */
 
+					/*
+					 * As streaming replication delay is too much, if
+					 * prefer_lower_delay_standby is true then elect new
+					 * load balance node which is lowest delayed,
+					 * false then send to the primary.
+					 */
+					if (STREAM &&
+						pool_config->delay_threshold &&
+						bkinfo->standby_delay > pool_config->delay_threshold)
+					{
+						ereport(DEBUG1,
+								(errmsg("could not load balance because of too much replication delay"),
+								 errdetail("destination = %d for query= \"%s\"", dest, query)));
+
+						if (pool_config->prefer_lower_delay_standby)
+						{
+							int new_load_balancing_node = select_load_balancing_node();
+
+							session_context->load_balance_node_id = new_load_balancing_node;
+							session_context->query_context->load_balance_node_id = session_context->load_balance_node_id;
+							pool_set_node_to_be_sent(query_context, session_context->query_context->load_balance_node_id);
+						}
+						else
+						{
+							pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
+						}
+					}
+
 					/*
 					 * If system catalog is used in the SELECT, we prefer to
 					 * send to the primary. Example: SELECT * FROM pg_class
@@ -579,7 +565,7 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 					 * primary system catalog. Please note that this test must
 					 * be done *before* test using pool_has_temp_table.
 					 */
-					if (pool_has_system_catalog(node))
+					else if (pool_has_system_catalog(node))
 					{
 						ereport(DEBUG1,
 								(errmsg("could not load balance because systems catalogs are used"),
@@ -640,44 +626,11 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 					else
 					{
 						if (pool_config->statement_level_load_balance)
-						{
 							session_context->load_balance_node_id = select_load_balancing_node();
-							bkinfo = pool_get_node_info(session_context->load_balance_node_id);
-						}
 
-						/*
-						 * As streaming replication delay is too much, if
-						 * prefer_lower_delay_standby is true then elect new
-						 * load balance node which is lowest delayed,
-						 * false then send to the primary.
-						 */
-						if (STREAM &&
-							pool_config->delay_threshold &&
-							bkinfo->standby_delay > pool_config->delay_threshold)
-						{
-							ereport(DEBUG1,
-									(errmsg("could not load balance because of too much replication delay"),
-									 errdetail("destination = %d for query= \"%s\"", dest, query)));
-
-							if (pool_config->prefer_lower_delay_standby)
-							{
-								int new_load_balancing_node = select_load_balancing_node();
-
-								session_context->load_balance_node_id = new_load_balancing_node;
-								session_context->query_context->load_balance_node_id = session_context->load_balance_node_id;
-								pool_set_node_to_be_sent(query_context, session_context->query_context->load_balance_node_id);
-							}
-							else
-							{
-								pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
-							}
-						}
-						else
-						{
-							session_context->query_context->load_balance_node_id = session_context->load_balance_node_id;
-							pool_set_node_to_be_sent(query_context,
-													 session_context->query_context->load_balance_node_id);
-						}
+						session_context->query_context->load_balance_node_id = session_context->load_balance_node_id;
+						pool_set_node_to_be_sent(query_context,
+												 session_context->query_context->load_balance_node_id);
 					}
 				}
 				else
@@ -693,10 +646,6 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 			}
 		}
 	}
-	else if (REPLICATION && query_context->is_multi_statement)
-	{
-		pool_setall_node_to_be_sent(query_context);
-	}
 	else if (REPLICATION)
 	{
 		if (pool_config->load_balance_mode &&
@@ -807,9 +756,24 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 	}
 
 	/*
-	 * DEALLOCATE or EXECUTE?
+	 * EXECUTE?
 	 */
-	if (IsA(node, DeallocateStmt) || IsA(node, ExecuteStmt))
+	if (IsA(node, ExecuteStmt))
+	{
+		POOL_SENT_MESSAGE *msg;
+
+		msg = pool_get_sent_message('Q', ((ExecuteStmt *) node)->name, POOL_SENT_MESSAGE_CREATED);
+		if (!msg)
+			msg = pool_get_sent_message('P', ((ExecuteStmt *) node)->name, POOL_SENT_MESSAGE_CREATED);
+		if (msg)
+			pool_copy_prep_where(msg->query_context->where_to_send,
+								 query_context->where_to_send);
+	}
+
+	/*
+	 * DEALLOCATE?
+	 */
+	else if (IsA(node, DeallocateStmt))
 	{
 		where_to_send_deallocate(query_context, node);
 	}
@@ -850,9 +814,9 @@ pool_send_and_wait(POOL_QUERY_CONTEXT * query_context,
 
 	/*
 	 * If the query is BEGIN READ WRITE or BEGIN ... SERIALIZABLE in
-	 * streaming replication mode, we send BEGIN to standbys instead.
-	 * The original_query which is BEGIN READ WRITE is sent to primary.
-	 * The rewritten_query BEGIN is sent to standbys.
+	 * native replication mode, we send BEGIN to standbys instead.
+	 * original_query which is BEGIN READ WRITE is sent to primary.
+	 * rewritten_query which is BEGIN is sent to standbys.
 	 */
 	if (pool_need_to_treat_as_if_default_transaction(query_context))
 	{
@@ -883,8 +847,8 @@ pool_send_and_wait(POOL_QUERY_CONTEXT * query_context,
 			continue;
 
 		/*
-		 * If we are in streaming replication mode or logical replication mode,
-		 * we do not send COMMIT/ABORT to standbys if it's in I (idle) state.
+		 * If in native replication mode, we do not send COMMIT/ABORT to
+		 * standbys if it's in I(idle) state.
 		 */
 		if (is_commit && MAIN_REPLICA && !IS_MAIN_NODE_ID(i) && TSTATE(backend, i) == 'I')
 		{
@@ -917,7 +881,6 @@ pool_send_and_wait(POOL_QUERY_CONTEXT * query_context,
 		}
 
 		per_node_statement_log(backend, i, string);
-		per_node_statement_notice(backend, i, string);
 		stat_count_up(i, query_context->parse_tree);
 		send_simplequery_message(CONNECTION(backend, i), len, string, MAJOR(backend));
 	}
@@ -962,15 +925,8 @@ pool_send_and_wait(POOL_QUERY_CONTEXT * query_context,
 		 * Check if some error detected.  If so, emit log. This is useful when
 		 * invalid encoding error occurs. In this case, PostgreSQL does not
 		 * report what statement caused that error and make users confused.
-		 * Also set reset_query_error to true in ERROR case. This does
-		 * anything in normal query processing but when processing reset
-		 * queries, this is important because it might mean DISCARD ALL
-		 * command fails. If so, we need to discard the connection cache so
-		 * that any session object (i.e. named statement) does not remain in
-		 * the last session.
 		 */
-		if (per_node_error_log(backend, i, string, "pool_send_and_wait: Error or notice message from backend", true) == 'E')
-			reset_query_error = true;
+		per_node_error_log(backend, i, string, "pool_send_and_wait: Error or notice message from backend: ", true);
 	}
 
 	return POOL_CONTINUE;
@@ -1011,9 +967,9 @@ pool_extended_send_and_wait(POOL_QUERY_CONTEXT * query_context,
 
 	/*
 	 * If the query is BEGIN READ WRITE or BEGIN ... SERIALIZABLE in
-	 * streaming replication mode, we send BEGIN to standbys instead.
-	 * The original_query which is BEGIN READ WRITE is sent to primary.
-	 * The rewritten_query BEGIN is sent to standbys.
+	 * native replication mode, we send BEGIN to standbys instead.
+	 * original_query which is BEGIN READ WRITE is sent to primary.
+	 * rewritten_query which is BEGIN is sent to standbys.
 	 */
 	if (pool_need_to_treat_as_if_default_transaction(query_context))
 	{
@@ -1102,7 +1058,6 @@ pool_extended_send_and_wait(POOL_QUERY_CONTEXT * query_context,
 			}
 
 			per_node_statement_log(backend, i, msgbuf);
-			per_node_statement_notice(backend, i, msgbuf);
 		}
 
 		/* if Execute message, count up stats count */
@@ -1181,7 +1136,7 @@ pool_extended_send_and_wait(POOL_QUERY_CONTEXT * query_context,
 			 * does not report what statement caused that error and make users
 			 * confused.
 			 */
-			per_node_error_log(backend, i, str, "pool_send_and_wait: Error or notice message from backend", true);
+			per_node_error_log(backend, i, str, "pool_send_and_wait: Error or notice message from backend: ", true);
 		}
 	}
 
@@ -1405,19 +1360,8 @@ static POOL_DEST send_to_where(Node *node, char *query)
 			}
 			/* SAVEPOINT related commands are sent to both primary and standby */
 			else if (is_savepoint_query(node))
-			{
-				if (SL_MODE && is_tx_started_by_multi_statement_query())
-				{
-					/*
-					 * But in streaming replication mode, if a transaction was
-					 * started by a multi statement query, SAVEPOINT should be
-					 * sent to primary because the transaction was started on
-					 * primary only.
-					*/
-					return POOL_PRIMARY;
-				}
 				return POOL_BOTH;
-			}
+
 			/*
 			 * 2PC commands
 			 */
@@ -1448,16 +1392,16 @@ static POOL_DEST send_to_where(Node *node, char *query)
 				{
 					A_Const    *v = (A_Const *) lfirst(list_item);
 
-					switch (v->val.type)
+					switch (nodeTag(&v->val))
 					{
 						case T_String:
-							if (!strcasecmp(v->val.val.str, "off") ||
-								!strcasecmp(v->val.val.str, "f") ||
-								!strcasecmp(v->val.val.str, "false"))
+							if (!strcasecmp(v->val.sval.sval, "off") ||
+								!strcasecmp(v->val.sval.sval, "f") ||
+								!strcasecmp(v->val.sval.sval, "false"))
 								ret = POOL_PRIMARY;
 							break;
 						case T_Integer:
-							if (v->val.val.ival)
+							if (v->val.ival.ival)
 								ret = POOL_PRIMARY;
 						default:
 							break;
@@ -1495,7 +1439,7 @@ static POOL_DEST send_to_where(Node *node, char *query)
 					{
 						bool		read_only;
 
-						read_only = ((A_Const *) opt->arg)->val.val.ival;
+						read_only = ((A_Const *) opt->arg)->val.ival.ival;
 						if (!read_only)
 							return POOL_PRIMARY;
 					}
@@ -1576,55 +1520,26 @@ static POOL_DEST send_to_where(Node *node, char *query)
 	return POOL_PRIMARY;
 }
 
-/*
- * Decide where to send given message.
- * "node" must be a parse tree of either DEALLOCATE or EXECUTE.
- */
 static
 void
 where_to_send_deallocate(POOL_QUERY_CONTEXT * query_context, Node *node)
 {
-	DeallocateStmt *d = NULL;
-	ExecuteStmt *e = NULL;
-	char		*name;
+	DeallocateStmt *d = (DeallocateStmt *) node;
 	POOL_SENT_MESSAGE *msg;
 
-	if (IsA(node, DeallocateStmt))
-	{
-		d = (DeallocateStmt *) node;
-		name = d->name;
-	}
-	else if (IsA(node, ExecuteStmt))
-	{
-		e = (ExecuteStmt *) node;
-		name = e->name;
-	}
-	else
-	{
-		ereport(ERROR,
-				(errmsg("invalid node type for where_to_send_deallocate")));
-		return;
-	}
-
 	/* DEALLOCATE ALL? */
-	if (d && (name == NULL))
+	if (d->name == NULL)
 	{
-		/* send to all backend node */
 		pool_setall_node_to_be_sent(query_context);
-		return;
 	}
-
-	/* ordinary DEALLOCATE or EXECUTE */
 	else
 	{
-		/* check if message was created by SQL PREPARE */
-		msg = pool_get_sent_message('Q', name, POOL_SENT_MESSAGE_CREATED);
+		msg = pool_get_sent_message('Q', d->name, POOL_SENT_MESSAGE_CREATED);
 		if (!msg)
-			/* message may be created by Parse message */
-			msg = pool_get_sent_message('P', name, POOL_SENT_MESSAGE_CREATED);
+			msg = pool_get_sent_message('P', d->name, POOL_SENT_MESSAGE_CREATED);
 		if (msg)
 		{
-			/* Inherit same map from PREPARE or Parse */
+			/* Inherit same map from PREPARE or PARSE */
 			pool_copy_prep_where(msg->query_context->where_to_send,
 								 query_context->where_to_send);
 
@@ -1632,33 +1547,8 @@ where_to_send_deallocate(POOL_QUERY_CONTEXT * query_context, Node *node)
 			query_context->load_balance_node_id = msg->query_context->load_balance_node_id;
 		}
 		else
-		{
-			/*
-			 * prepared statement was not found.
-			 * There are two cases when this could happen.
-			 * (1) mistakes by client. In this case backend will return ERROR
-			 * anyway.
-			 * (2) previous query was issued as multi-statement query. e.g.
-			 * SELECT 1\;PREPARE foo AS SELECT 1;
-			 * In this case pgpool does not know anything about the prepared
-			 * statement "foo".
-			 */
-			if (SL_MODE)
-			{
-				/*
-				 * In streaming replication or logical replication, sent to
-				 * primary node only.
-				 */
-				pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
-			}
-			else
-			{
-				/*
-				 * In other mode, sent to all node.
-				 */
-				pool_setall_node_to_be_sent(query_context);
-			}
-		}
+			/* prepared statement was not found */
+			pool_setall_node_to_be_sent(query_context);
 	}
 }
 
@@ -1728,10 +1618,10 @@ is_set_transaction_serializable(Node *node)
 		{
 			A_Const    *v = (A_Const *) lfirst(list_item);
 
-			switch (v->val.type)
+			switch (nodeTag(&v->val))
 			{
 				case T_String:
-					if (!strcasecmp(v->val.val.str, "serializable"))
+					if (!strcasecmp(v->val.sval.sval, "serializable"))
 						return true;
 					break;
 				default:
@@ -1756,7 +1646,7 @@ is_set_transaction_serializable(Node *node)
 			{
 				A_Const    *v = (A_Const *) opt->arg;
 
-				if (!strcasecmp(v->val.val.str, "serializable"))
+				if (!strcasecmp(v->val.sval.sval, "serializable"))
 					return true;
 			}
 		}
@@ -1798,7 +1688,7 @@ is_read_write(TransactionStmt *node)
 		{
 			bool		read_only;
 
-			read_only = ((A_Const *) opt->arg)->val.val.ival;
+			read_only = ((A_Const *) opt->arg)->val.ival.ival;
 			if (read_only)
 				return false;	/* TRANSACTION READ ONLY */
 			else
@@ -1833,8 +1723,8 @@ is_serializable(TransactionStmt *node)
 
 		if (!strcmp("transaction_isolation", opt->defname) &&
 			IsA(opt->arg, A_Const) &&
-			((A_Const *) opt->arg)->val.type == T_String &&
-			!strcmp("serializable", ((A_Const *) opt->arg)->val.val.str))
+			IsA(&((A_Const *) opt->arg)->val, String) &&
+			!strcmp("serializable", ((A_Const *) opt->arg)->val.sval.sval))
 			return true;
 	}
 	return false;
@@ -1842,10 +1732,10 @@ is_serializable(TransactionStmt *node)
 
 /*
  * If the query is BEGIN READ WRITE or
- * BEGIN ... SERIALIZABLE in streaming replication mode,
+ * BEGIN ... SERIALIZABLE in native replication mode,
  * we send BEGIN to standbys instead.
- * The original_query which is BEGIN READ WRITE is sent to primary.
- * The rewritten_query BEGIN is sent to standbys.
+ * original_query which is BEGIN READ WRITE is sent to primary.
+ * rewritten_query which is BEGIN is sent to standbys.
  */
 bool
 pool_need_to_treat_as_if_default_transaction(POOL_QUERY_CONTEXT * query_context)
@@ -2117,16 +2007,16 @@ pool_is_transaction_read_only(Node *node)
 		{
 			A_Const    *v = (A_Const *) lfirst(list_item);
 
-			switch (v->val.type)
+			switch (nodeTag(&v->val))
 			{
 				case T_String:
-					if (!strcasecmp(v->val.val.str, "on") ||
-						!strcasecmp(v->val.val.str, "t") ||
-						!strcasecmp(v->val.val.str, "true"))
+					if (!strcasecmp(v->val.sval.sval, "on") ||
+						!strcasecmp(v->val.sval.sval, "t") ||
+						!strcasecmp(v->val.sval.sval, "true"))
 						ret = true;
 					break;
 				case T_Integer:
-					if (v->val.val.ival)
+					if (v->val.ival.ival)
 						ret = true;
 				default:
 					break;
@@ -2152,7 +2042,7 @@ pool_is_transaction_read_only(Node *node)
 			{
 				bool		read_only;
 
-				read_only = ((A_Const *) opt->arg)->val.val.ival;
+				read_only = ((A_Const *) opt->arg)->val.ival.ival;
 				if (read_only)
 				{
 					ret = true;
