@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2023	PgPool Global Development Group
+ * Copyright (c) 2003-2022	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -29,10 +29,6 @@
 #include <signal.h>
 #include <stdio.h>
 
-#ifndef PGPOOL_ADM
-#include "parser/pg_config_manual.h"
-#endif
-
 /*
  * startup packet definitions (v2) stolen from PostgreSQL
  */
@@ -53,6 +49,7 @@
 #define MAX_CONNECTION_SLOTS MAX_NUM_BACKENDS
 #define MAX_DB_HOST_NAMELEN	 MAX_FDQN_HOSTNAME_LEN
 #define MAX_PATH_LENGTH 256
+#define NAMEDATALEN 64
 
 typedef enum
 {
@@ -103,6 +100,7 @@ typedef struct
 	bool		quarantine;		/* true if node is CON_DOWN because of
 								 * quarantine */
 	uint64		standby_delay;	/* The replication delay against the primary */
+	bool		standby_delay_by_time;	/* true if standby_delay is measured in microseconds, not bytes */
 	SERVER_ROLE role;			/* Role of server. used by pcp_node_info and
 								 * failover() to keep track of quarantined
 								 * primary node */
@@ -126,7 +124,8 @@ typedef enum
 	WAIT_FOR_CONNECT,
 	COMMAND_EXECUTE,
 	IDLE,
-	IDLE_IN_TRANS
+	IDLE_IN_TRANS,
+	CONNECTING
 }			ProcessStatus;
 
 /*
@@ -178,10 +177,13 @@ typedef struct
 										 * this process */
 	int			client_connection_count;	/* how many times clients used this process */
 	ProcessStatus	status;
-	char		need_to_restart;	/* If non 0, exit this child process as
+	bool		need_to_restart;	/* If non 0, exit this child process as
 									 * soon as current session ends. Typical
 									 * case this flag being set is failback a
 									 * node in streaming replication mode. */
+	bool		exit_if_idle;
+	int		pooled_connections; /* Total number of pooled connections
+									  * by this child */
 }			ProcessInfo;
 
 /*
