@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2024	PgPool Global Development Group
+ * Copyright (c) 2003-2022	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -82,6 +82,7 @@ static bool connect_using_existing_connection(POOL_CONNECTION * frontend,
 								  POOL_CONNECTION_POOL * backend,
 								  StartupPacket *sp);
 static void check_restart_request(void);
+static void check_exit_request(void);
 static void enable_authentication_timeout(void);
 static void disable_authentication_timeout(void);
 static int	wait_for_new_connections(int *fds, SockAddr *saddr);
@@ -98,6 +99,8 @@ static bool backend_cleanup(POOL_CONNECTION * volatile *frontend, POOL_CONNECTIO
 static void child_will_go_down(int code, Datum arg);
 static int opt_sort(const void *a, const void *b);
 
+static bool unix_fds_not_isset(int* fds, int num_unix_fds, fd_set* opt);
+
 /*
  * Non 0 means SIGTERM (smart shutdown) or SIGINT (fast shutdown) has arrived
  */
@@ -137,11 +140,6 @@ struct timeval startTime;
 bool		stop_now = false;
 #endif
 
-/*
- * If true, connection pool has been initialized.
- */
-static bool connection_pool_initialized = false;
-
 /*
 * child main loop
 */
@@ -225,7 +223,6 @@ do_child(int *fds)
 	{
 		child_exit(POOL_EXIT_AND_RESTART);
 	}
-	connection_pool_initialized = true;
 
 	/*
 	 * Open pool_passwd in child process.  This is necessary to avoid the file
@@ -304,7 +301,7 @@ do_child(int *fds)
 			pool_close(child_frontend);
 			child_frontend = NULL;
 		}
-
+		update_pooled_connection_count();
 		MemoryContextSwitchTo(TopMemoryContext);
 		FlushErrorState();
 	}
@@ -329,6 +326,7 @@ do_child(int *fds)
 		/* pgpool stop request already sent? */
 		check_stop_request();
 		check_restart_request();
+		check_exit_request();
 		accepted = 0;
 		/* Destroy session context for just in case... */
 		pool_session_context_destroy();
@@ -349,6 +347,10 @@ do_child(int *fds)
 		if (front_end_fd == RETRY)
 			continue;
 
+		set_process_status(CONNECTING);
+		/* Reset any exit if idle request even if it's pending */
+		pool_get_my_process_info()->exit_if_idle = false;
+
 		con_count = connection_count_up();
 
 		if (pool_config->reserved_connections > 0)
@@ -463,7 +465,7 @@ do_child(int *fds)
 
 		/* Mark this connection pool is not connected from frontend */
 		pool_coninfo_unset_frontend_connected(pool_get_process_context()->proc_id, pool_pool_index());
-
+		update_pooled_connection_count();
 		accepted = 0;
 		connection_count_down();
 		if (pool_config->log_disconnections)
@@ -487,6 +489,8 @@ do_child(int *fds)
 	child_exit(POOL_EXIT_NO_RESTART);
 }
 
+
+
 /* -------------------------------------------------------------------
  * private functions
  * -------------------------------------------------------------------
@@ -508,7 +512,7 @@ backend_cleanup(POOL_CONNECTION * volatile *frontend, POOL_CONNECTION_POOL * vol
 	sp = MAIN_CONNECTION(backend)->sp;
 
 	/*
-	 * cache connection if connection cache configuration parameter is enabled
+	 * cach connection if connection cache configuration parameter is enabled
 	 * and frontend connection is not invalid
 	 */
 	if (sp && pool_config->connection_cache != 0 && frontend_invalid == false)
@@ -538,10 +542,8 @@ backend_cleanup(POOL_CONNECTION * volatile *frontend, POOL_CONNECTION_POOL * vol
 	if (cache_connection)
 	{
 		/*
-		 * For those special databases, or when frontend client exits
-		 * abnormally, we don't cache connection to backend.  Also we have to
-		 * check whether reset query failed. If so, the existing connection to
-		 * backend may not be used and we don't want to cache the connection.
+		 * For those special databases, and when frontend client exits
+		 * abnormally, we don't cache connection to backend.
 		 */
 		if ((sp &&
 			 (!strcmp(sp->database, "template0") ||
@@ -550,8 +552,7 @@ backend_cleanup(POOL_CONNECTION * volatile *frontend, POOL_CONNECTION_POOL * vol
 			  !strcmp(sp->database, "regression"))) ||
 			(*frontend != NULL &&
 			 ((*frontend)->socket_state == POOL_SOCKET_EOF ||
-			  (*frontend)->socket_state == POOL_SOCKET_ERROR)) ||
-			reset_query_error)
+			  (*frontend)->socket_state == POOL_SOCKET_ERROR)))
 			cache_connection = false;
 	}
 
@@ -727,7 +728,7 @@ read_startup_packet(POOL_CONNECTION * cp)
 		case 1234:				/* cancel or SSL request */
 			/* set dummy database, user info */
 			sp->database = palloc0(1);
-			sp->user = palloc0(1);
+			sp->user = palloc(1);
 			break;
 
 		default:
@@ -751,11 +752,8 @@ read_startup_packet(POOL_CONNECTION * cp)
 	}
 
 	/* The database defaults to their user name. */
-	if (sp->database == NULL)
-		sp->database = pstrdup(sp->user);
-	else if (sp->database[0] == '\0')
+	if (sp->database == NULL || sp->database[0] == '\0')
 	{
-		pfree(sp->database);
 		sp->database = pstrdup(sp->user);
 	}
 
@@ -1034,8 +1032,9 @@ static POOL_CONNECTION_POOL * connect_backend(StartupPacket *sp, POOL_CONNECTION
 
 		for (i = 0; i < NUM_BACKENDS; i++)
 		{
-			if (VALID_BACKEND(i) && CONNECTION_SLOT(backend, i))
+			if (VALID_BACKEND(i))
 			{
+
 				/* set DB node id */
 				pool_set_db_node_id(CONNECTION(backend, i), i);
 
@@ -1157,7 +1156,6 @@ static RETSIGTYPE close_idle_connection(int sig)
 	POOL_CONNECTION_POOL *p = pool_connection_pool;
 	ConnectionInfo *info;
 	int			save_errno = errno;
-	int			main_node_id;
 
 	/*
 	 * DROP DATABASE is ongoing.
@@ -1165,35 +1163,44 @@ static RETSIGTYPE close_idle_connection(int sig)
 	if (ignore_sigusr1)
 		return;
 
+#ifdef NOT_USED
+	ereport(DEBUG1,
+			(errmsg("close connection request received")));
+#endif
+
 	for (j = 0; j < pool_config->max_pool; j++, p++)
 	{
-		main_node_id = in_use_backend_id(p);
-		if (main_node_id < 0)
+		if (!MAIN_CONNECTION(p))
+			continue;
+		if (!MAIN_CONNECTION(p)->sp)
+			continue;
+		if (MAIN_CONNECTION(p)->sp->user == NULL)
 			continue;
 
-		if (!CONNECTION_SLOT(p, main_node_id))
-			continue;
-		if (!CONNECTION_SLOT(p, main_node_id)->sp)
-			continue;
-		if (CONNECTION_SLOT(p, main_node_id)->sp->user == NULL)
-			continue;
-
-		if (CONNECTION_SLOT(p, main_node_id)->closetime > 0)	/* idle connection? */
+		if (MAIN_CONNECTION(p)->closetime > 0)	/* idle connection? */
 		{
-			bool	freed = false;
+#ifdef NOT_USED
+			ereport(DEBUG1,
+					(errmsg("closing idle connection"),
+					 errdetail("user: %s database: %s", MAIN_CONNECTION(p)->sp->user, MAIN_CONNECTION(p)->sp->database)));
+#endif
 
 			pool_send_frontend_exits(p);
 
 			for (i = 0; i < NUM_BACKENDS; i++)
 			{
-				if (!CONNECTION_SLOT(p, i))
+				if (!VALID_BACKEND(i))
 					continue;
 
-				if (!freed)
+				if (i == 0)
 				{
+					/*
+					 * only first backend allocated the memory for the start
+					 * up packet
+					 */
 					pool_free_startup_packet(CONNECTION_SLOT(p, i)->sp);
 					CONNECTION_SLOT(p, i)->sp = NULL;
-					freed = true;
+
 				}
 				pool_close(CONNECTION(p, i));
 			}
@@ -1302,7 +1309,7 @@ child_will_go_down(int code, Datum arg)
 	}
 
 	/* let backend know now we are exiting */
-	if (connection_pool_initialized)
+	if (pool_connection_pool)
 		close_all_backend_connections();
 }
 void
@@ -1433,6 +1440,23 @@ pool_initialize_private_backend_status(void)
 	my_main_node_id = REAL_MAIN_NODE_ID;
 }
 
+static void
+check_exit_request(void)
+{
+	/*
+	 * Check if exit request is set because of spare children management.
+	 */
+	if (pool_get_my_process_info()->exit_if_idle)
+	{
+		ereport(LOG,
+				(errmsg("Exit flag set"),
+				 errdetail("Exiting myself")));
+
+		pool_get_my_process_info()->exit_if_idle = 0;
+		child_exit(POOL_EXIT_NO_RESTART);
+	}
+}
+
 static void
 check_restart_request(void)
 {
@@ -1446,7 +1470,7 @@ check_restart_request(void)
 				(errmsg("failover or failback event detected"),
 				 errdetail("restarting myself")));
 
-		pool_get_my_process_info()->need_to_restart = 0;
+		pool_get_my_process_info()->need_to_restart = false;
 		child_exit(POOL_EXIT_AND_RESTART);
 	}
 }
@@ -1667,7 +1691,7 @@ retry_accept:
 	 * Set no delay if AF_INET socket. Not sure if this is really necessary
 	 * but PostgreSQL does this.
 	 */
-	if (!FD_ISSET(fds[0], &rmask))	/* fds[0] is UNIX domain socket */
+	if (unix_fds_not_isset(fds, pool_config->num_unix_socket_directories, &rmask))
 	{
 		on = 1;
 		if (setsockopt(afd, IPPROTO_TCP, TCP_NODELAY,
@@ -1698,6 +1722,20 @@ retry_accept:
 	return afd;
 }
 
+static bool
+unix_fds_not_isset(int* fds, int num_unix_fds, fd_set* opt)
+{
+	int		i;
+	for (i = 0; i < num_unix_fds; i++)
+	{
+		if (!FD_ISSET(fds[i], opt))
+			continue;
+
+		return false;
+	}
+	return true;
+}
+
 static void
 check_config_reload(void)
 {
@@ -1711,10 +1749,9 @@ check_config_reload(void)
 		if (pool_config->enable_pool_hba)
 		{
 			load_hba(get_hba_file_name());
+			if (strcmp("", pool_config->pool_passwd))
+				pool_reopen_passwd_file();
 		}
-		if (strcmp("", pool_config->pool_passwd))
-			pool_reopen_passwd_file();
-
 		got_sighup = 0;
 	}
 }
