@@ -3,7 +3,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2025	PgPool Global Development Group
+ * Copyright (c) 2003-2022	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -146,14 +146,6 @@ do_worker_child(void)
 	/* Initialize per process context */
 	pool_init_process_context();
 
-	/*
-	 * Open pool_passwd.
-	 */
-	if (strcmp("", pool_config->pool_passwd))
-	{
-		pool_reopen_passwd_file();
-	}
-
 	if (sigsetjmp(local_sigjmp_buf, 1) != 0)
 	{
 		pool_signal(SIGALRM, SIG_IGN);
@@ -168,15 +160,9 @@ do_worker_child(void)
 
 	for (;;)
 	{
-		WD_STATES	wd_status;
-
 		MemoryContextSwitchTo(WorkerMemoryContext);
 		MemoryContextResetAndDeleteChildren(WorkerMemoryContext);
-		/*
-		 * Since WorkerMemoryContext is used for "slots", we need to clear it
-		 * so that new slots are allocated later on.
-		 */
-		memset(slots, 0, sizeof(slots));
+		WD_STATES	wd_status;
 
 		CHECK_REQUEST;
 
@@ -203,7 +189,7 @@ do_worker_child(void)
 			Req_info->switching == false)
 		{
 			/*
-			 * Acquire follow primary lock. If fail to acquire lock, try again.
+			 * Acquire follow primary lock. If fail to acqure lock, try again.
 			 */
 			follow_primary_lock_acquired = false;
 
@@ -333,13 +319,15 @@ check_replication_time_lag(void)
 	int			i;
 	POOL_SELECT_RESULT *res;
 	POOL_SELECT_RESULT *res_rep;	/* query results of pg_stat_replication */
-	unsigned long long int lsn[MAX_NUM_BACKENDS];
+	uint64	lsn[MAX_NUM_BACKENDS];
 	char	   *query;
 	char	   *stat_rep_query;
 	BackendInfo *bkinfo;
-	unsigned long long int lag;
+	uint64	lag;
+	uint64	delay_threshold_by_time;
 	ErrorContextCallback callback;
 	int		active_standby_node;
+	bool	replication_delay_by_time;
 
 	/* clear replication state */
 	for (i = 0; i < NUM_BACKENDS; i++)
@@ -362,20 +350,6 @@ check_replication_time_lag(void)
 		return;
 	}
 
-	if (!VALID_BACKEND(REAL_PRIMARY_NODE_ID))
-	{
-		/*
-		 * No need to check replication delay if primary is down. This could
-		 * happen if ALWAYS_PRIMARY flag is on because REAL_PRIMARY_NODE_ID
-		 * macro returns the node id which ALWAYS_PRIMARY flag is set to.  If
-		 * we do not check this, subsequent test (i == PRIMARY_NODE_ID) in the
-		 * for loop below will return unexpected result because
-		 * PRIMARY_NODE_ID macro returns MAIN_NODE_ID, which could be a
-		 * standby server.
-		 */
-		return;
-	}
-
 	/*
 	 * Register a error context callback to throw proper context message
 	 */
@@ -385,6 +359,7 @@ check_replication_time_lag(void)
 	error_context_stack = &callback;
 	stat_rep_query = NULL;
 	active_standby_node = 0;
+	replication_delay_by_time = false;
 
 	for (i = 0; i < NUM_BACKENDS; i++)
 	{
@@ -404,7 +379,7 @@ check_replication_time_lag(void)
 
 		if (server_version[i] == 0)
 		{
-			query = "SELECT pg_catalog.current_setting('server_version_num')";
+			query = "SELECT current_setting('server_version_num')";
 
 			/*
 			 * Get backend server version. If the query fails, keep previous
@@ -422,21 +397,27 @@ check_replication_time_lag(void)
 		if (PRIMARY_NODE_ID == i)
 		{
 			if (server_version[i] >= PG10_SERVER_VERSION)
-				query = "SELECT pg_catalog.pg_current_wal_lsn()";
+				query = "SELECT pg_current_wal_lsn()";
 			else
-				query = "SELECT pg_catalog.pg_current_xlog_location()";
+				query = "SELECT pg_current_xlog_location()";
 
 			if (server_version[i] == PG91_SERVER_VERSION)
-				stat_rep_query = "SELECT application_name, state, '' AS sync_state FROM pg_catalog.pg_stat_replication";
+				stat_rep_query = "SELECT application_name, state, '' AS sync_state, '' AS replay_lag FROM pg_stat_replication";
+			else if (server_version[i] >= PG10_SERVER_VERSION)
+			{
+				stat_rep_query = "SELECT application_name, state, sync_state,(EXTRACT(EPOCH FROM replay_lag)*1000000)::integer FROM pg_stat_replication";
+				if (pool_config->delay_threshold_by_time > 0)
+					replication_delay_by_time = true;
+			}
 			else if (server_version[i] > PG91_SERVER_VERSION)
-				stat_rep_query = "SELECT application_name, state, sync_state FROM pg_catalog.pg_stat_replication";
+				stat_rep_query = "SELECT application_name, state, sync_state, '' AS replay_lag FROM pg_stat_replication";
 		}
 		else
 		{
 			if (server_version[i] >= PG10_SERVER_VERSION)
-				query = "SELECT pg_catalog.pg_last_wal_replay_lsn()";
+				query = "SELECT pg_last_wal_replay_lsn()";
 			else
-				query = "SELECT pg_catalog.pg_last_xlog_replay_location()";
+				query = "SELECT pg_last_xlog_replay_location()";
 
 			active_standby_node++;
 		}
@@ -477,10 +458,11 @@ check_replication_time_lag(void)
 			{
 				int		j;
 				char	*s;
+#define	NUM_COLS 4
 
 				for (j = 0; j < res_rep->numrows; j++)
 				{
-					if (strcmp(res_rep->data[j*3], bkinfo->backend_application_name) == 0)
+					if (strcmp(res_rep->data[j*NUM_COLS], bkinfo->backend_application_name) == 0)
 					{
 						/*
 						 * If sr_check_user has enough privilege, it should return
@@ -488,10 +470,21 @@ check_replication_time_lag(void)
 						 * res_rep->data[1] and [2]. So we need to prepare for the
 						 * latter case.
 						 */
-						s = res_rep->data[j*3+1]? res_rep->data[j*3+1] : "";
+						s = res_rep->data[j*NUM_COLS+1]? res_rep->data[j*NUM_COLS+1] : "";
 						strlcpy(bkinfo->replication_state, s, NAMEDATALEN);
-						s = res_rep->data[j*3+2]? res_rep->data[j*3+2] : "";
+
+						s = res_rep->data[j*NUM_COLS+2]? res_rep->data[j*NUM_COLS+2] : "";
 						strlcpy(bkinfo->replication_sync_state, s, NAMEDATALEN);
+
+						s = res_rep->data[j*NUM_COLS+3];
+						if (s)
+						{
+							bkinfo->standby_delay = atol(s);
+							ereport(LOG,
+									(errmsg("standby_delay: %lu", bkinfo->standby_delay)));
+						}
+						else
+							bkinfo->standby_delay = 0;
 					}
 				}
 			}
@@ -515,17 +508,52 @@ check_replication_time_lag(void)
 		}
 		else
 		{
-			bkinfo->standby_delay = lag;
+			if (replication_delay_by_time)
+			{
+				/*
+				 * If replication delay is measured by time, indicate it in
+				 * shared memory area.
+				 */
+				bkinfo->standby_delay_by_time = true;
+			}
+			else
+			{
+				/*
+				 * If replication delay is not measured by time, set the LSN
+				 * lag to shared memory area.
+				 */
+				bkinfo->standby_delay = lag;
+				bkinfo->standby_delay_by_time = false;
+			}
 
 			/* Log delay if necessary */
-			if ((pool_config->log_standby_delay == LSD_ALWAYS && lag > 0) ||
-				(pool_config->delay_threshold &&
-				 pool_config->log_standby_delay == LSD_OVER_THRESHOLD &&
-				 lag > pool_config->delay_threshold))
+			if (replication_delay_by_time)
 			{
-				ereport(LOG,
-						(errmsg("Replication of node:%d is behind %llu bytes from the primary server (node:%d)",
-								i, lsn[PRIMARY_NODE_ID] - lsn[i], PRIMARY_NODE_ID)));
+				lag = bkinfo->standby_delay;
+				delay_threshold_by_time = pool_config->delay_threshold_by_time;
+				delay_threshold_by_time *= 1000000;
+
+				/* Log delay if necessary */
+				if ((pool_config->log_standby_delay == LSD_ALWAYS && lag > 0) ||
+					(pool_config->log_standby_delay == LSD_OVER_THRESHOLD &&
+					 lag > delay_threshold_by_time))
+				{
+					ereport(LOG,
+							(errmsg("Replication of node: %d is behind %.6f second(s) from the primary server (node: %d)",
+									i, ((float)lag)/1000000, PRIMARY_NODE_ID)));
+				}
+			}
+			else
+			{
+				if ((pool_config->log_standby_delay == LSD_ALWAYS && lag > 0) ||
+					(pool_config->delay_threshold &&
+					 pool_config->log_standby_delay == LSD_OVER_THRESHOLD &&
+					 lag > pool_config->delay_threshold))
+				{
+					ereport(LOG,
+							(errmsg("Replication of node: %d is behind " UINT64_FORMAT " bytes from the primary server (node: %d)",
+									i, (uint64)(lsn[PRIMARY_NODE_ID] - lsn[i]), PRIMARY_NODE_ID)));
+				}
 			}
 		}
 	}
@@ -620,10 +648,6 @@ reload_config(void)
 	MemoryContextSwitchTo(oldContext);
 	if (pool_config->enable_pool_hba)
 		load_hba(get_hba_file_name());
-
-	if (strcmp("", pool_config->pool_passwd))
-		pool_reopen_passwd_file();
-
 	reload_config_request = 0;
 }
 
